@@ -3,29 +3,35 @@ from typing import List, Optional
 
 import typer
 from rich import print
-from rich.table import Table
-from rich.console import Console
-from rich import box
-from rich.panel import Panel
-from pathlib import Path
-
 from rich.console import Console
 
 from compose_companion.configurations import Configurations
 
 from compose_companion.functions import (
     container_down,
+    container_stop,
+    container_start,
     container_pause,
     container_up,
-    server_down,
-    server_pause,
-    server_up,
+    print_config
 )
 
+
 app = typer.Typer()
 err_console = Console(stderr=True)
 
 
+@app.callback(invoke_without_command=True)
+def main(ctx: typer.Context):
+    """
+    A companion for Docker Compose.
+    """
+    if not ctx.invoked_subcommand:
+        ctx.get_help()
+    else:
+        Configurations()
+
+
 @app.command()
 def up(
     targets: Optional[List[str]] = typer.Argument(
@@ -38,17 +44,47 @@ def up(
         "-r",
         help="Recreates containers even if their configuration and image haven't changed. Toggles docker compose --force-recreate option.",
     ),
+    build: bool = typer.Option(
+        False,
+        "--build",
+        "-b",
+        help="Build images before starting containers. Toggles docker compose --build option.",
+    ),
 ):
     """
     Starts up targeted containers.
     If no target is defined, starts all containers.
-    Equivalent to `docker compose up -d`.
+    Calls to `docker compose up -d --remove-orphans`.
     """
+    services = Configurations().services
     if not targets:
-        server_up(recreate)
+        for container in services:
+            container_up(container, recreate, build)
         return
     for container in targets:
-        container_up(container, recreate)
+        container_up(container, recreate, build)
+    return
+
+
+@app.command()
+def start(
+    targets: Optional[List[str]] = typer.Argument(
+        None,
+        help="Containers in which to run the command. If no targets are passed, runs for all containers.",
+    ),
+):
+    """
+    Starts targeted containers.
+    If no target is defined, starts all containers.
+    Equivalent to `docker compose start`.
+    """
+    services = Configurations().services
+    if not targets:
+        for container in services:
+            container_start(container)
+        return
+    for container in targets:
+        container_start(container)
 
 
 @app.command()
@@ -61,21 +97,49 @@ def pause(
     """
     Pauses targeted containers.
     If no target is defined, pauses all containers.
-    Equivalent to `docker compose stop`.
+    Equivalent to `docker compose pause`.
     """
+    services = Configurations().services
     if not targets:
-        server_pause()
+        for container in services:
+            container_pause(container)
         return
     for container in targets:
         container_pause(container)
 
 
+@app.command()
+def stop(
+    targets: Optional[List[str]] = typer.Argument(
+        None,
+        help="Containers in which to run the command. If no targets are passed, runs for all containers.",
+    ),
+):
+    """
+    Stops targeted containers.
+    If no target is defined, stops all containers.
+    Equivalent to `docker compose stop`.
+    """
+    services = Configurations().services
+    if not targets:
+        for container in services:
+            container_stop(container)
+        return
+    for container in targets:
+        container_stop(container)
+
+
 @app.command()
 def down(
     targets: Optional[List[str]] = typer.Argument(
         None,
         help="Containers in which to run the command. If no targets are passed, runs for all containers.",
     ),
+    remove_volumes: Optional[bool] = typer.Option(
+        False,
+        "--remove-volumes",
+        help="Remove any anonymous volumes attached to the containers. Equivalent do docker compose -v/--volumes option.",
+    ),
     confirmation: bool = typer.Option(
         False,
         "-f",
@@ -87,16 +151,18 @@ def down(
     """
     Shuts down target containers.
     If no target is defined, shuts down all containers.
-    Equivalent to `docker compose rm -sf` or `docker compose down`.
+    Equivalent to `docker compose rm -sf`.
     """
+    services = Configurations().services
 
     if not confirmation:
         return
     if not targets:
-        server_down()
+        for container in services:
+            container_down(container, remove_volumes)
         return
     for container in targets:
-        container_down(container)
+        container_down(container, remove_volumes)
 
 
 @app.command()
@@ -157,10 +223,38 @@ def exec(
     compose_file = Configurations().compose_file
 
     if not command:
-        run(f"docker compose -f {compose_file} exec -d={detach} sh")
+        run(f"docker compose -f {compose_file} exec -d={detach} {target} sh")
         return
     run(
-        f"docker compose -f {compose_file} exec -d={detach} {target}  {' '.join(command)}"
+        f"docker compose -f {compose_file} exec -d={detach} {target} {' '.join(command)}"
+    )
+
+
+@app.command()
+def ps(
+    targets: Optional[List[str]] = typer.Argument(
+        None,
+        help="Containers in which to run the command. If no targets are passed, runs for all containers.",
+    ),
+    include_stopped: Optional[bool] = typer.Option(
+        False,
+        "-a",
+        "--all",
+        help="Show all, including stopped containers. Equivalent to docker compose -a/--all option.",
+    ),
+):
+    """
+    Lists containers.
+    If no target is defined, lists all containers.
+    Equivalent to `docker compose ps`.
+    """
+    compose_file = Configurations().compose_file
+
+    if not targets:
+        run(f"docker compose -f {compose_file} ps")
+        return
+    run(
+        f"docker compose -f {compose_file} ps {' '.join(targets)} -a={include_stopped}"
     )
 
 
@@ -181,36 +275,20 @@ def config(
     conf = Configurations()
 
     if key and key in conf.config_keys:
-        conf.set_config(key, value) if value else print(conf.get_config(key))
+        if (key == 'compose_file' and value):
+            conf.compose_file = value
+        elif (key == 'companion_file' and value):
+            conf.companion_file = value
+        print(conf.get_config(key))
         return
 
     if key and key not in conf.config_keys:
         err_console.print(
-            "Invalid Key. To view all available configurations run the command without any arguments.\n"
-        )
-        return
-
-    grid = Table.grid(expand=True)
-    grid.add_column("Key", style="cyan")
-    grid.add_column("Value", style="magenta")
-    for key in conf.config_keys:
-        grid.add_row(
-            key,
-            conf.get_config(key),
+            "Invalid Key. Here are the available keys:\n"
         )
 
-    panel = Panel(
-        grid,
-        title="Configurations",
-        title_align="left",
-        border_style="dim",
-    )
-    print(panel)
+    print_config()
 
 
-@app.callback(invoke_without_command=True)
-def main():
-    """
-    A companion for Docker Compose.
-    """
-    Configurations()
+if __name__ == "__main__":
+    app()
